<?xml version='1.0' encoding='UTF-8'?>
<!DOCTYPE xsl:stylesheet >
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="html"/>

<xsl:template match="/qjml">
    <xsl:variable name="root"><xsl:value-of select="@root"/></xsl:variable>
    <html>

    <head>
        <title><xsl:value-of select="$root"/> QJML Schema</title>
    </head>

    <body bgcolor="#ffffcc"> 
    <center><h2><xsl:value-of select="$root"/> QJML Schema</h2></center>
    <br/>
    <table width="100%"><tr>
    <td><a href="#TREE">SCHEMA TREE</a></td>
    <td><a href="#INDEX">INDEX</a></td>
    </tr></table>
    <br/>
    
    <xsl:for-each select="include">
        <hr/>
        This QJML file includes <xsl:value-of select="."/>
        <br/>
    </xsl:for-each>
    
    <xsl:for-each select="excludes">
        <hr/>
        Not to be included: <xsl:value-of select="."/>
        <xsl:if test="@ID">
            (ID=<xsl:value-of select="@ID"/>)
        </xsl:if>
        <br/>
    </xsl:for-each>
    
    <xsl:apply-templates select="interface | abstract | text | bean | link"/>
    
    <hr/>
    <a name="TREE"/>
    <h3>SCHEMA TREE</h3>
    <pre>
    <xsl:for-each select="*[@label=$root or (not(@label) and @tag=$root)]">
        <xsl:call-template name="print-tree-elem">
            <xsl:with-param name="depth"></xsl:with-param>
            <xsl:with-param name="done"></xsl:with-param>
        </xsl:call-template>
    </xsl:for-each>
    </pre>
    
    <hr/>
    <a name="INDEX"/>
    <h3>INDEX</h3>
    <xsl:for-each select="interface | abstract | text | bean | link">
        <xsl:sort select="@label|@tag"/>
        <xsl:choose>
            <xsl:when test="@label">
                <a href="#{@label}"><xsl:value-of select="@label"/></a>
                <xsl:if test="@tag">
                    (tag=<xsl:value-of select="@tag"/>)
                </xsl:if>
            </xsl:when>
            <xsl:otherwise>
                <a href="#{@tag}"><xsl:value-of select="@tag"/></a>
            </xsl:otherwise>
        </xsl:choose>
        <br/>
    </xsl:for-each>
    </body>
    </html>
</xsl:template>

<xsl:template name="print-tree-elem">
    <xsl:param name="depth"/>
    <xsl:param name="done"/>
    <xsl:variable name="newdone"><xsl:value-of select="concat($done,@label|@tag,',')"/></xsl:variable>
    <a name="tree_{@label|@tag}"/>
    <xsl:value-of select="$depth"/>
    <xsl:choose>
        <xsl:when test="@label != ''">
            <a href="#{@label}"><xsl:value-of select="@label"/></a>
            <xsl:if test="@tag">
                (tag=<xsl:value-of select="@tag"/>)
            </xsl:if>
        </xsl:when>
        <xsl:otherwise>
            <a href="#{@tag}"><xsl:value-of select="@tag"/></a>
        </xsl:otherwise>
    </xsl:choose>
    <xsl:text>
</xsl:text>
    <xsl:if test="not(contains($done,concat(@label|@tag,',')))">
        <xsl:for-each select=".//elements//item">
            <xsl:call-template name="print-tree-child">
                <xsl:with-param name="depth" select="$depth"/>
                <xsl:with-param name="done" select="$newdone"/>
                <xsl:with-param name="last" select="position()=last()"/>
            </xsl:call-template>
        </xsl:for-each>
        <xsl:variable name="elname"><xsl:value-of select="@label|@tag"/></xsl:variable>
        <xsl:for-each select="/qjml/*[extends=$elname or implements=$elname]">
            <xsl:variable name="newdepth">
                <xsl:choose>
                    <xsl:when test="position() = last()">
                        <xsl:value-of select =
    "concat(translate($depth,'->','&#160;&#160;'),'&#160;---')"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select =
    "concat(translate($depth,'->','&#160;&#160;'),'|---')"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:call-template name="print-tree-elem">
                <xsl:with-param name="depth" select="$newdepth"/>
                <xsl:with-param name="done" select="$newdone"/>
            </xsl:call-template>
        </xsl:for-each>
    </xsl:if>
</xsl:template>

<xsl:template name="print-tree-child">
    <xsl:param name="depth"/>
    <xsl:param name="done"/>
    <xsl:param name="last"/>
    <xsl:variable name="newdepth">
        <xsl:choose>
            <xsl:when test="$last">
                <xsl:value-of select =
               "concat(translate($depth,'->','&#160;&#160;'),'&#160;---')"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select =
               "concat(translate($depth,'->','&#160;&#160;'),'|---')"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <xsl:variable name="elname"><xsl:choose>
        <xsl:when test="@cref">
            <xsl:variable name="cref" select="@cref"/>
            <xsl:value-of select="/qjml//item[@label=$cref]/@coin"/>
        </xsl:when>
        <xsl:otherwise>
            <xsl:value-of select="@coin"/>
        </xsl:otherwise>
    </xsl:choose></xsl:variable>
    <xsl:for-each select="/qjml/*[@label=$elname or (not(@label) and @tag=$elname)]">
        <xsl:call-template name="print-tree-elem">
            <xsl:with-param name="depth" select="$newdepth"/>
            <xsl:with-param name="done" select="$done"/>
        </xsl:call-template>
    </xsl:for-each>
</xsl:template>

<xsl:template match="interface|abstract|text|bean|link">
    <hr/>
    <xsl:choose>
        <xsl:when test="@label != ''">
            <a name="{@label}"/>
        </xsl:when>
        <xsl:when test="@tag != ''">
            <a name="{@tag}"/>
        </xsl:when>
    </xsl:choose>
    <h3><xsl:value-of select="name()"/>: 
    <xsl:if test="@wild != ''">
        <xsl:if test="@wild = 'True'">
            wild 
        </xsl:if>
    </xsl:if>

    <font color="red"><xsl:value-of select="@tag|@label"/></font>
    
    <xsl:if test="@label and @tag">
        (tag=<xsl:value-of select="@tag"/>)
    </xsl:if>
    
    <xsl:if test="@ID">
        (ID=<xsl:value-of select="@ID"/>)
    </xsl:if>
    
    </h3>
    
    <xsl:for-each select="extends">
        extends <font color="red"><a href="#{.}">
        <xsl:value-of select="."/></a></font>
        <br/>
    </xsl:for-each>
    
    <xsl:for-each select="implements">
        implements <font color="red"><a href="#{.}">
        <xsl:value-of select="."/></a></font>
        <br/>
    </xsl:for-each>
    
    <xsl:if test="@schema">
        schema: <tt><xsl:value-of select="@schema"/></tt>
        <br/>
    </xsl:if>
    
    <xsl:variable name="elname"><xsl:value-of select="@label|@tag"/></xsl:variable>
    
    <xsl:variable name="elems" select="/qjml/*[.//elements//item[@coin=$elname]]"/>
    
    <xsl:if test="$elems">
        <xsl:variable name="ellabels"><xsl:for-each select="$elems">
            <xsl:if test="elements//item[@coin=$elname]/@label">
                <xsl:value-of select="concat(' (',elements//item[@coin=$elname]/@label,') ')"/>
            </xsl:if>
        </xsl:for-each></xsl:variable>
    
        child of:
        <xsl:for-each select="$elems | /qjml/*[.//elements//item[contains($ellabels,concat('(',@cref,')'))]]">
            <xsl:text> </xsl:text>
            <a href="#{@label|@tag}"><xsl:value-of select="@label|@tag"/></a>
            <xsl:text> </xsl:text>
        </xsl:for-each>
        <br/>
    </xsl:if>
    
    <xsl:variable name="atts" select="/qjml/*[.//attributes//item[@coin=$elname]]"/>
    
    <xsl:if test="$atts">
        <xsl:variable name="attlabels"><xsl:for-each select="$atts">
            <xsl:if test="attributes//item[@coin=$elname]/@label">
                <xsl:value-of select="concat(' (',attributes//item[@coin=$elname]/@label,') ')"/>
            </xsl:if>
        </xsl:for-each></xsl:variable>

        attribute of:
        <xsl:for-each select="$atts | /qjml/*[.//attributes//item[contains($attlabels,concat('(',@cref,')'))]]">
            <xsl:text> </xsl:text>
            <a href="#{@label|@tag}"><xsl:value-of select="@label|@tag"/></a>
            <xsl:text> </xsl:text>
        </xsl:for-each>
        <br/>
    </xsl:if>
    
    <xsl:if test="@type != ''">
        content: <tt><xsl:value-of select="@type"/></tt><br/> 
    </xsl:if>
    
    <xsl:apply-templates select="targetClass | targetEditor | targetFactory"/>
    
    <xsl:if test="rem != ''">
        <br/>
        <xsl:apply-templates select="rem"/>
        <br/>
    </xsl:if>
    
    <xsl:if test="@generate != ''">
        <xsl:if test="@generate = 'False'">
            will not be generated with QJML2Java<br/> 
        </xsl:if>
    </xsl:if>
    
    <xsl:if test="@validInherited = 'True'">
        enums and ranges are inherited<br/>
    </xsl:if>
    
    <xsl:if test="enum">
        <br/>
        valid values:<ul>
        <xsl:for-each select="enum">
            <li><tt><xsl:value-of select="@value"/></tt>
            <xsl:if test="@ID">
                (ID=<xsl:value-of select="@ID"/>)
            </xsl:if>
            <xsl:if test="rem">
                <br/>
                <xsl:apply-templates select="rem"/>
            </xsl:if>
            </li>
        </xsl:for-each>
        </ul>
    </xsl:if>
    
    <xsl:apply-templates select="range"/>
    
    <xsl:if test="@inline">
        force the display to a new line:
        <tt><xsl:value-of select="@inline"/></tt><br/>
    </xsl:if>
    
    <xsl:if test="@trim">
        Remove leading/trailing whitespace:
        <tt><xsl:value-of select="@trim"/></tt><br/>
    </xsl:if>
    
    <xsl:if test="attributes/item != ''">
        <h3>Attributes
        <xsl:if test="attributes/@inherited = 'False'">
            (the attributes are not inherited)
        </xsl:if>
        <xsl:if test="attributes/@ID">
            (ID=<xsl:value-of select="attributes/@ID"/>)
        </xsl:if>
        </h3>
        <ul>
        <xsl:apply-templates select="attributes/item"/>
        </ul>
        <br/>
    </xsl:if>
    
    <xsl:if test="elements/item or elements/selection or elements/sequence">
        <h3>Children
        <xsl:if test="elements/@ID">
            (ID=<xsl:value-of select="elements/@ID"/>)
        </xsl:if>
        </h3>
        <a href="#tree_{@tag}">full tree</a>
        <ul>
        <xsl:apply-templates select="elements/*"/>
        </ul>
        <br/>
    </xsl:if>
    <xsl:if test="field | property">
        <h3>Other fields/properties</h3>
        <ul>
        <xsl:for-each select="field | property">
            <li><xsl:apply-templates select="."/></li>
        </xsl:for-each>
        </ul>
        <br/>
    </xsl:if>
    <xsl:if test="/qjml/*[extends=$elname or implements=$elname]">
        <h3>Is extended/implemented by</h3>
        <ul>
        <xsl:for-each select="/qjml/*[extends=$elname or implements=$elname]">
            <li><a href="#{@label|@tag}"><xsl:value-of select="@label|@tag"/></a></li>
        </xsl:for-each>
        </ul>
        <br/>
    </xsl:if>
</xsl:template>

<xsl:template match="item">
    <xsl:if test="@label!=''">
        <a name="{@label}"/>
    </xsl:if>
    
    <li>
    
    <xsl:choose>
        <xsl:when test="../attributes">
            <font color="green"><xsl:value-of select="@coin"/></font>
        </xsl:when>
        <xsl:when test="@coin!=''">
            <a href="#{@coin}"><xsl:value-of select="@coin"/></a>
        </xsl:when>
        <xsl:when test="@cref!=''">
            <a href="#{@cref}"><xsl:value-of select="@cref"/></a>
        </xsl:when>
    </xsl:choose>
    
    <xsl:if test="not(@cref)">
        <xsl:text> (</xsl:text>
        <xsl:choose>
            <xsl:when test="@optional='True'">
                optional
            </xsl:when>
            <xsl:otherwise>
                required
            </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="@repeating='True'">
            , repeating
        </xsl:if>
        )
    </xsl:if>
    
    <xsl:if test="@ID">
        (ID=<xsl:value-of select="@ID"/>)
    </xsl:if>
    
    <xsl:if test="@schema">
        <br/>
        schema: <tt><xsl:value-of select="@schema"/></tt>
    </xsl:if>
    
    <xsl:if test="@type!=''">
        <br/>
        type: <tt><xsl:value-of select="@type"/></tt>
    </xsl:if>
    
    <xsl:if test="rem!=''">
        <br/>
        <xsl:apply-templates select="rem"/>
        <br/>
    </xsl:if>
    
    <xsl:if test="@value!=''">
        <br/>
        default value: <tt><xsl:value-of select="@value"/></tt>
    </xsl:if>
    
    <xsl:if test="@fixed = 'True'">
        <br/>
        Fixed value.
    </xsl:if>
    
    <xsl:if test="@generate != ''">
        <xsl:if test="@generate = 'False'">
            <br/>will not be generated with QJML2Java
        </xsl:if>
    </xsl:if>
    
    <xsl:if test="aref">
        accessor: <tt><a href="#accessor_{aref}"><xsl:value-of select="aref"/></a></tt>
    </xsl:if>
    
    <xsl:for-each select="field | property | id | identity | ignore | accessClass">
        <br/>
        <xsl:apply-templates select="."/>
    </xsl:for-each>
    
    <xsl:if test="id">
        <br/>
        ID
    </xsl:if>
    
    <br/>
    </li>
</xsl:template>

<xsl:template match="selection">
    <xsl:if test="@label!=''">
        <a name="{@label}"/>
    </xsl:if>
    <li>selection
    <xsl:text> (</xsl:text>
    <xsl:choose>
        <xsl:when test="@optional='True'">
            optional
        </xsl:when>
        <xsl:otherwise>
            required
        </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="@repeating='True'">
        , repeating
    </xsl:if>
    )
    <xsl:if test="@cref!=''">
        <br/><a href="#{@cref}"><xsl:value-of select="@cref"/></a>
    </xsl:if>
    <xsl:apply-templates select="field | property | id | identity | ignore | accessClass"/>
    <ul>
    <xsl:apply-templates select="*[self::item or self::selection or self::sequence]"/>
    </ul></li>
</xsl:template>

<xsl:template match="sequence">
    <xsl:if test="@label!=''">
        <a name="{@label}"/>
    </xsl:if>
    <li>sequence
    <xsl:text> (</xsl:text>
    <xsl:choose>
        <xsl:when test="@optional='True'">
            optional
        </xsl:when>
        <xsl:otherwise>
            required
        </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="@repeating='True'">
        , repeating
    </xsl:if>
    )
    <xsl:if test="@cref!=''">
        <br/><a href="#{@cref}"><xsl:value-of select="@cref"/></a>
    </xsl:if>
    <xsl:apply-templates select="field | property | id | identity | ignore | accessClass"/>
    <ul>
    <xsl:apply-templates select="*[self::item or self::selection or self::sequence]"/>
    </ul></li>
</xsl:template>

<xsl:template match="targetClass">
    targetClass: <tt><xsl:value-of select="."/></tt>
    <xsl:if test="@ID">
        (ID=<xsl:value-of select="@ID"/>)
    </xsl:if>
    <br/> 
</xsl:template>

<xsl:template match="targetEditor">
    targetEditor: <tt><xsl:value-of select="@editor"/></tt>
    <xsl:if test="@ID">
        (ID=<xsl:value-of select="@ID"/>)
    </xsl:if>
    <br/>
    <xsl:if test="@tetarget">
         tetarget: <tt><xsl:value-of select="@tetarget"/></tt><br/> 
    </xsl:if>
</xsl:template>

<xsl:template match="targetFactory">
    targetFactory: <tt><xsl:value-of select="."/></tt>
    <xsl:if test="@ID">
        (ID=<xsl:value-of select="@ID"/>)
    </xsl:if>
    <br/>
    <xsl:if test="@tetarget">
         tetarget: <tt><xsl:value-of select="@tetarget"/></tt><br/> 
    </xsl:if>
</xsl:template>
    
<xsl:template match="rem">
    <xsl:value-of disable-output-escaping="yes" select="."/>
    <xsl:if test="@ID">
        (ID=<xsl:value-of select="@ID"/>)
    </xsl:if>
    <xsl:text>
</xsl:text>
</xsl:template>

<xsl:template match="cref">
    <li>cref: <tt><a href="#{.}"><xsl:value-of select="."/></a></tt></li>
</xsl:template>

<xsl:template match="field">
    <xsl:if test="@name!=''">
        <a name="accessor_{@name}"/>
    </xsl:if>
    field: <tt><xsl:value-of select="@name"/></tt>
    <xsl:call-template name="access_attributes"/>
</xsl:template>

<xsl:template match="property">
    <xsl:if test="@name!=''">
        <a name="accessor_{@name}"/>
    </xsl:if>
    property: <tt><xsl:value-of select="@name"/></tt>
    <xsl:call-template name="access_attributes"/>
</xsl:template>

<xsl:template match="id">
    The data item is an identifier maintained by QDoc
    <xsl:if test="@ID">
        (ID=<xsl:value-of select="@ID"/>)
    </xsl:if>
</xsl:template>

<xsl:template match="identity">
    The data item is the entire coin
    <xsl:if test="@kind">
        (kind: <tt><xsl:value-of select="@kind"/></tt>)
    </xsl:if>
</xsl:template>

<xsl:template match="ignore">
    The data item is ignored
    <xsl:if test="@ID">
        (ID=<xsl:value-of select="@ID"/>)
    </xsl:if>
</xsl:template>

<xsl:template match="accessClass">
    accessClass: <xsl:value-of select="@accessor"/>
</xsl:template>

<xsl:template name="access_attributes">
    <xsl:if test="@kind">
        (kind: <tt><xsl:value-of select="@kind"/></tt>)
    </xsl:if>

    <xsl:if test="@content">
        , content is described by:
        <tt><a href="#{@content}"><xsl:value-of select="@content"/></a></tt>
    </xsl:if>

    <xsl:if test="@size">
        , size: <tt><xsl:value-of select="@size"/></tt>
    </xsl:if>

    <xsl:if test="@key">
        content accessor:
        , <tt><a href="#accessor_{@key}"><xsl:value-of select="@key"/></a></tt>
    </xsl:if>
    
    <xsl:if test="@coin">
        , data is defined by:
        <tt><a href="#{@coin}"><xsl:value-of select="@coin"/></a></tt>
    </xsl:if>
    
    <xsl:if test="@ID">
        , ID=<xsl:value-of select="@ID"/>
    </xsl:if>
    
    <xsl:if test="@initializer">
        , initializer: <xsl:value-of select="@initializer"/>
    </xsl:if>
    
    <xsl:if test="@abstract">
        , abstract: <xsl:value-of select="@abstract"/>
    </xsl:if>
    
    <xsl:if test="@read">
        , read: <xsl:value-of select="@read"/>
    </xsl:if>
    
    <xsl:if test="@write">
        , write: <xsl:value-of select="@write"/>
    </xsl:if>
</xsl:template>

<xsl:template match="range">
    <br/>
    range of valid values:<ul>
    <xsl:if test="@imin">
        <li>inclusive minimum:<tt><xsl:value-of select="@imin"/></tt></li>
    </xsl:if>
    <xsl:if test="@xmin">
        <li>exclusive minimum:<tt><xsl:value-of select="@xmin"/></tt></li>
    </xsl:if>
    <xsl:if test="@imax">
        <li>inclusive maximum:<tt><xsl:value-of select="@imax"/></tt></li>
    </xsl:if>
    <xsl:if test="@xmax">
        <li>exclusive maximum:<tt><xsl:value-of select="@xmax"/></tt></li>
    </xsl:if>
    </ul>
    <xsl:apply-templates select="rem"/>
    <br/>
</xsl:template>

</xsl:stylesheet>
